// main.cpp
//
// Copyright (C) 2012, ARM Research Labs, LLC.
// See www.armresearch.com for the copyright terms.
//
// Create and maintain the sniffer configuration file for SNFMilter.
//
// The configuration file and instructions are specified on the
// command line, and are used to create/modify the configuration file,
// and integrate or unintegrate with the specified MTA.
//
//
// $Id$
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>
#include <sys/stat.h>

#include <exception>
#include <iostream>
#include <fstream>

#include "SNFMulti.hpp"
#include "SNFMilterConfig.hpp"

using namespace std;

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Configuration. ////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

/// Version string.
const char* SNF_MILTERCONFIG_VERSION = "SNFMilterConfig 0.0.1 Build: " __DATE__ " " __TIME__;

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// End of configuration. /////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[]) {

    SNFMilterConfig SnfMilterConfig;

    if (!SnfMilterConfig.GetCommandLineInput(argc, argv) ||                     // If our command line arguments
        SnfMilterConfig.Help()) {                                               // don't look right, or if help is
                                                                                // requested  then display our help
        SnfMilterConfig.DisplayHelp(SNF_MILTERCONFIG_VERSION);                  // screen.
	return 0;

    }

    bool DebugMode = false;                                                     // This will be our debug mode.
    string argv0(argv[0]);                                                      // Capture how we were called.
    if(
      string::npos != argv0.find("Debug") ||                                    // If we find "Debug" or
      string::npos != argv0.find("debug")                                       // "debug" in our command path
      ) {                                                                       // then we are in DebugMode.
        DebugMode = true;                                                       // Set the flag and tell the
        cout << SNF_MILTERCONFIG_VERSION << endl;                               // watchers.
        cout << "Debug Mode" << endl;
    }

    try {                                                                       // Catch anything that breaks loose.

        SnfMilterConfig.UpdateConfigFiles();                                    // Create config files if they don't
                                                                                // exist, or update config files.

	SnfMilterConfig.DoIntegrationCommand();                                 // Integrate/unintegrate.

    }                                                                           // That's all folks.

    catch(exception& e) {                                                       // Report any normal exceptions.
        cerr << "SNFMilterConfig Exception: " << e.what() << endl;
    }
    catch (snfCFGmgr::LoadFailure) {                                            // Error loading configuration file.
	cerr << "snfCFGmgr Exception:  Unable to load the configuration file "
	     << SnfMilterConfig.GetConfigFileName() << endl;
    }
    catch(...) {                                                                // Report any unexpected exceptions.
        cerr << "SNFMilterConfig Panic! Unknown Exception!" << endl;
    }

    return 0;                                                                   // Normally we return zero.
}
